#!/usr/bin/python
portnum = 1080				#The port number
maxlen = 30					#The maximum number of posts to send per call
file_enabled = True			#Whether the server is to allow files
file_extensions = []	#The allowed extensions. If [] then all are allowed
file_maxsize = 2e6			#The maximum file size to allow (bytes)


print "Starting up SciRoom..."

#Include the twisted magic
from twisted.web.server import Site
from twisted.web.static import File
from twisted.web.resource import Resource
from twisted.internet import reactor

import cgi		#Allows for sending posts to and fro
import sqlite3	#All website info is stored in a sqlite database
import hashlib	#For sending hashed IPs as identifiers
import json		#For wrapping up the sent data in json format

"""
Database setup - the database is created if none exists and is initialized to be
ready for use
"""
db = None
try:	#Check for existence of the database
	db = open('./sciroom.db','r')
	db.close()
	db = sqlite3.connect('./sciroom.db')
except:
	#The database does not exist
	db = sqlite3.connect('./sciroom.db')
	cur = db.cursor()
	#From top to bottom:
	#	posts: the place where all the text is stored
	#	banned: a list of IPs that are banned from the server
	cur.execute('CREATE TABLE posts (chat text, usr text, ip text, date text, postnum integer, txt BLOB)')
	cur.execute('CREATE TABLE banned (ip text, reason text)')
	db.commit()
	cur.close()
#db.close()


"""
The following code is the server-side of SciRoom - AKA, the content
"""

class Settings(Resource):
	def render_GET(self,request):
		global maxlen,file_enabled,file_extensions,file_maxsize
		try:
			room = cgi.escape(request.args["rid"][0])
			print "%s: ROOM (%s)" % (request.getClientIP(),room)
			cur = db.cursor()
			if (cur.execute('SELECT COUNT (*) FROM banned WHERE ip=?',(request.getClientIP(),)).fetchall()[0][0] != 0):
				cur.close()
				return json.dumps({'banned':True})
			cur.close()
			return json.dumps({
				'maxposts': maxlen,
				'file_enabled': file_enabled,
				'file_ext': file_extensions,
				'file_maxsize':file_maxsize
				})
		except:
			return ''

class Check(Resource): 
	def render_GET(self,request):
		global db, maxlen
		try:
			rm=cgi.escape(request.args["rid"][0])
			pnum=cgi.escape(request.args["pnum"][0])
			c = db.cursor()
			c.execute('SELECT * FROM posts WHERE (chat=? AND postnum>=?) ORDER BY postnum',(rm,pnum,))
			arr=[]
			for row in c:
				m = hashlib.md5()
				m.update(row[2])
				arr.append({'usr':row[1],'pwd':m.hexdigest(),'date':row[3],'text':row[5]})
				if (len(arr) >= maxlen): break	#only return maxlen rows per call
			c.close()
			if (arr==[]):
				return ""
			else:
				print "%s: Get from '%s' at %s, %i posts" % (request.getClientIP(),rm,pnum,len(arr))
			return json.dumps(arr)	#Dumps the data into json, and sends it on
		except:
			return ""
	def render_POST(self, request):
		global db
		cur = db.cursor()
		if (cur.execute('SELECT COUNT (*) FROM banned WHERE ip=?',(request.getClientIP(),)).fetchall()[0][0] != 0):
			cur.close()
			print "%s: BANNED IP post attempt" % request.getClientIP()
			return ''	#This IP is banned!
		cur.close()
		c = db.cursor()
		#Obtains the post number
		num = c.execute('SELECT COUNT(*) FROM posts WHERE chat=?',(cgi.escape(request.args["rid"][0]),)).fetchall()[0][0]
		#Creates the data tuple
		try:
			t=(
				cgi.escape(request.args["rid"][0]),	#roomID
				cgi.escape(request.args["usr"][0]),	#Username
				request.getClientIP(),				#The IP address of client
				cgi.escape(request.args["date"][0]),#The date
				num,								#The post number
				cgi.escape(request.args["txt"][0]),	#The post text
			)
			c.execute('INSERT INTO posts values (?,?,?,?,?,?)',t)
		except:
			print "Failed to post"
			return ""
		c.close()
		print "%s: Posted in '%s'" % (request.getClientIP(),cgi.escape(request.args["rid"][0]))
		return ""	#The post does not check for new posts, since it is happening at a high rate already

"""
Here we have the server setup
"""

sciroom = File('./www')				#Run it on the SciRoom directory

#The following code adds the variable content to the static directory structure
sciroom.putChild("check",Check())
sciroom.putChild("settings",Settings())

#Sets up the server to run
factory = Site(sciroom)
reactor.listenTCP(portnum,factory)
print "Sciroom is running on port %s" % portnum
reactor.run()
#Clean up after shutdown
db.commit()
db.close()
print "\nShut Down successfully\n"
